/*
 *  Lucidor - e-book reader
 *  Copyright © 2008-2013  Mikael Ylikoski
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

Components.utils.import("chrome://lucidor/content/utils.jsm");

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var prefs;
var old_bookcase_file;
var new_bookcase_file;

function init() {
    prefs = Components.classes["@mozilla.org/preferences-service;1"].
	getService(Components.interfaces.nsIPrefService).
	getBranch("extensions.lucifox.");

    try {
	let file = prefs.getComplexValue("bookcase.folder",
					 Components.interfaces.nsIFile);
	//old_bookcase_file = file;
    } catch (e) {
	// FIXME
    }

    /* Font list */
    let pref = document.getElementById("font.name.serif.x-western");
    let current = pref.value;
    let flist = document.getElementById("font_list");
    let fpop = document.getElementById("font_popup");
    let fe = Components.classes["@mozilla.org/gfx/fontenumerator;1"]
	.getService(Components.interfaces.nsIFontEnumerator);
    let fonts = fe.EnumerateFonts("x-western", "", {});
    for (let f in loop(fonts)) {
	let item = document.createElementNS(XUL_NS, "menuitem");
	item.setAttribute("value", f);
	item.setAttribute("label", f);
	fpop.appendChild(item);
	if (f == current)
	    flist.value = f;
    }
    if (!flist.value && current) {
	let item = document.createElementNS(XUL_NS, "menuitem");
	item.setAttribute("value", current);
	item.setAttribute("label", current);
	fpop.appendChild(item);
	flist.value = current;
    }
}

var reload_time = -1;

function reload_style_later(event) {
    reload_time = (new Date()).getTime();
    setTimeout(maybe_reload_style, 1100);
}

function maybe_reload_style() {
    if (reload_time < (new Date()).getTime() - 1000)
	reload_style();
}

function reload_style() {
    let sss = Components.classes["@mozilla.org/content/style-sheet-service;1"].
	getService(Components.interfaces.nsIStyleSheetService);
    let ios = Components.classes["@mozilla.org/network/io-service;1"].
	getService(Components.interfaces.nsIIOService);
    let uri = ios.newURI("x-lucidor://css/", null, null);
    if (sss.sheetRegistered(uri, sss.USER_SHEET))
	sss.unregisterSheet(ios.newURI("x-lucidor://css/", null, null),
			    sss.USER_SHEET);
    sss.loadAndRegisterSheet(uri, sss.USER_SHEET);
}

function background_dec() {
    let picker = document.getElementById("backgroundPicker");
    picker.color = color_dec(picker.color);
}

function background_inc() {
    let picker = document.getElementById("backgroundPicker");
    picker.color = color_inc(picker.color);
}

function foreground_dec() {
    let picker = document.getElementById("foregroundPicker");
    picker.color = color_dec(picker.color);
}

function foreground_inc() {
    let picker = document.getElementById("foregroundPicker");
    picker.color = color_inc(picker.color);
}

function color_dec(color) {
    let red = parseInt(color.substring(1, 3), 16);
    let green = parseInt(color.substring(3, 5), 16);
    let blue = parseInt(color.substring(5, 7), 16);
    red -= 2;
    green -= 2;
    blue -= 2;
    red = Math.min(0xff, Math.max(0, red));
    green = Math.min(0xff, Math.max(0, green));
    blue = Math.min(0xff, Math.max(0, blue));
    return "#" + (0x1000000 + red * 0x10000 + green * 0x100 + blue).toString(16).substring(1);
}

function color_inc(color) {
    let red = parseInt(color.substring(1, 3), 16);
    let green = parseInt(color.substring(3, 5), 16);
    let blue = parseInt(color.substring(5, 7), 16);
    red += 2;
    green += 2;
    blue += 2;
    red = Math.min(0xff, Math.max(0, red));
    green = Math.min(0xff, Math.max(0, green));
    blue = Math.min(0xff, Math.max(0, blue));
    return "#" + (0x1000000 + red * 0x10000 + green * 0x100 + blue).toString(16).substring(1);
}

function syncfrom_use_doc_fonts() {
    let pref = document.getElementById("browser.display.use_document_fonts");
    return pref.value != 0;
}

function syncto_use_doc_fonts() {
    if (document.getElementById("useDocumentFonts").checked)
	return 1;
    return 0;
}

function clear_bookmarks() {
    let bookcase = Components.classes["@lucidor.org/bookcaseLiteService;1"].
	getService().wrappedJSObject;
    if (bookcase.open())
	bookcase.clear();
}

function clear_annotations() {
    let note_manager = Components.classes["@lucidor.org/notemanagerService;1"].
	getService().wrappedJSObject;
    if (note_manager.open())
	note_manager.clear();
}

function accept() {
    /*
    if (old_bookcase_file && new_bookcase_file &&
	!new_bookcase_file.equals(old_bookcase_file)) {
	let files = [];
	for (let book in bookcase) {
	    file = book...;
	    if (old_bookcase_file.contains(file, false))
		files.push(file);
	}
	if (files) {
	    let prompts = ...;
	    let text = "Your bookcase contains books that are stored in the old bookcase directory. Do you want to move them to the new bookcase directory?";
	    let migrate = prompts.confirm();
	    if (migrate) {
		for (let file in loop(files)) {
		    let name = "FIXME";
		    try {
			file.moveTo(new_bookcase_file, name);
		    } catch (e) {

		    }
		    // update bookcase book location
		}
	    }
	}
    }
    */
}

function select_directory() {
    let fp = Components.classes["@mozilla.org/filepicker;1"].
	createInstance(Components.interfaces.nsIFilePicker);
    let strbundle = document.getElementById("lucifox-strings");
    let title = strbundle.getString("select_path.title");
    fp.init(window, title, Components.interfaces.nsIFilePicker.modeGetFolder);

    let dir;
    try {
	dir = prefs.getComplexValue("bookcase.folder",
				    Components.interfaces.nsIFile);
	fp.displayDirectory = dir;
    } catch (e) {
    }

    let callback = {
	done: function(result) {
	    if (result != fp.returnOK)
		return;
	    prefs.setComplexValue("bookcase.folder",
				  Components.interfaces.nsIFile, fp.file);
	    //new_bookcase_file = fp.file;
	}
    };

    if (fp.open != undefined)
	fp.open(callback);
    else
	callback.done(fp.show());
}
